function boxs = buildbox(dim,sizes,steps)
% BOXS = BUILDBOX(DIM,SIZES,STEPS)
%
%   build list of boxes for image of size DIM.
%   SIZES: list of sizes (all boxes are square)
%   STEPS: sliding values (equal in both directions)
%   There is one step for each size, so both SIZES and STEPS are vectors
%   of equal length (if STEPS is scalar, it is understood that all sizes
%   have the same step value).
%   BOXS(:,i) is the i-th box, with corners in [BOXS(1,i),BOXS(2,i)] etc.
%   in order: tl, bl, tr, br
% 
%   NOTES: 
%   - the integral image will have one extra pixel in each dimension)
%   - coordinates are in the original image system. For example, a box
%     [2 2 6 2 2 7 6 7]' 
%     corresponds to image region I(2:6,2:7).

% (C) Greg Shakhnarovich, TTI-Chicago, 2007

if (nargin < 3)
  steps = floor(sizes/2);
elseif (length(steps)==1)
  steps = steps*ones(size(sizes));
end

boxs = [];

% build boxes
for s=1:length(sizes)
  % shifts of the top left:  horizontal
  shiftV = 1:steps(s):dim(1)-sizes(s)+1;
  n= length(shiftV);
  % vertical
  shiftH = 1:steps(s):dim(2)-sizes(s)+1;  
  m= length(shiftH);
  
  tl = [repmat(shiftV,1,m);
	reshape(repmat(shiftH,n,1),1,[])]; 
  bl = [tl(1,:)+sizes(s)-1; tl(2,:)]; 
  tr = [tl(1,:); tl(2,:)+sizes(s)-1];
  br = [tl(1,:)+sizes(s)-1; tl(2,:)+sizes(s)-1];
  
  boxs = [boxs [tl;bl;tr;br]];
end

